#!/usr/bin/env python

import sys
#inkscape_path = '/Applications/Local Applications/Inkscape.app/Contents/Resources/extensions'
#sys.path.append(inkscape_path)

# We will use inex module with predefined effect base class.
import inkex, cubicsuperpath, simplestyle, copy, math, re, bezmisc

def make_curve(last, csp, next, radius):
	
	if (csp[0] != csp[1] or csp[2] != csp[1]):
		# Not a flat corner
		return None
		
	if (csp == last or csp == next):
		# Zero-length segment
		return None

	pt = list(csp[1])
	vec0 = [pt[0] - last[1][0], pt[1] - last[1][1]]
	vec1 = [pt[0] - next[1][0], pt[1] - next[1][1]]
	len0 = math.hypot(*vec0)
	len1 = math.hypot(*vec1)
	if (len0 <= 0 or len1 <= 0):
		# Zero-length segment
		return None

	vec0 = [ val/len0 for val in vec0 ]
	vec1 = [ val/len1 for val in vec1 ]
	dotprod = (vec0[0]*vec1[0] + vec0[1]*vec1[1])
	rad = radius * (dotprod+1.0)
		
	vec0 = [ val*rad for val in vec0 ]
	vec1 = [ val*rad for val in vec1 ]
		
	cp0 = [pt[0] - vec0[0], pt[1] - vec0[1]]
	cp1 = [pt[0] - vec1[0], pt[1] - vec1[1]]
	cphalf0 = [pt[0] - 0.5*vec0[0], pt[1] - 0.5*vec0[1]]
	cphalf1 = [pt[0] - 0.5*vec1[0], pt[1] - 0.5*vec1[1]]
	
	return ([cp0,cp0,cphalf0], [cphalf1,cp1,cp1])


class DoIt(inkex.Effect):
	def __init__(self):
		inkex.Effect.__init__(self)
		self.OptionParser.add_option("-r", "--rad",
			action="store", type="float", 
			dest="rad", default=0.0,
			help="rounding radius")
			
	def effect(self):
		for id, node in self.selected.iteritems():
			if node.tagName == 'path':
				dnode = node.attributes.getNamedItem('d')
				path = cubicsuperpath.parsePath(dnode.value)
				#print >>sys.stderr, '###', dnode.value
				#print >>sys.stderr, '###', path

				newpath = []
				
				for subpath in path:
					newsub = []

					if (len(subpath) >= 3):
						if (subpath[-1] == subpath[-2]):
							del subpath[-1]

					closed = False
					mustclose = False
					if (len(subpath) >= 2):
						if (subpath[-1] == subpath[0]):
							closed = True
							del subpath[-1]
							
					lensub = len(subpath)
					
					for ix in range(lensub):
						csp = subpath[ix]
						if (ix <= 0 or ix >= lensub-1):
							if (not closed):
								newsub.append(csp)
								continue
							mustclose = True

						if (ix <= 0):
							last = subpath[-1]
						else:
							last = subpath[ix-1]
						if (ix >= lensub-1):
							next = subpath[0]
						else:
							next = subpath[ix+1]

						pair = make_curve(last, csp, next, self.options.rad)
						if (pair is None):
							newsub.append(csp)
							continue

						newsub.append(pair[0])
						newsub.append(pair[1])

					if (mustclose):
						newsub.append(newsub[0])
					newpath.append(newsub)
				
				dnode.value = cubicsuperpath.formatPath(newpath)

eff = DoIt()
eff.affect()
